/////////////////////////////////////////////////////////////////////
// WSCGenDlg.cpp : implementation file
//
// Main program dialog. This is the user interface since this is a 
// dialog based application.
/////////////////////////////////////////////////////////////////////
//SoundCard/Wavefile signal generator program
// Copyright 1999.    Moe Wheatley AE4JY  <ae4jy@mindspring.com>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "WSCGen.h"
#include "WSCGenDlg.h"
#include "ProcThread.h"
#include <math.h>
#include "Wave.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////  local defines
#define MAX_SNR 60.0
#define MIN_SNR -40.0
#define MIN_FREQ 10

#define BPSK_RMS_CONSTANT 1.1247// Fudge factor to make average BPSK signal RMS = 1
#define BPSK_CONSTANT (BPSK_RMS_CONSTANT/1.414)


#define RMS_AMPLITUDE 4739.0	//RMS amplitude out of 32768
								// pick so that worst case Gaussian
								// plus sin wave will not overflow soundcard
								// 32768 = RMS*5.5 + 1.414*RMS
								// RMS = 4739;
#define ID_TIMER 1
#define STATUS_IDLE 0
#define STATUS_RUNNING 1
#define STATUS_SAVING 2
#define STATUS_PROCFINISHED 3


CProcThread* vpProcThread;

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWSCGenDlg dialog

CWSCGenDlg::CWSCGenDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CWSCGenDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CWSCGenDlg)
	m_BWLimit = TRUE;
	m_PskIdleOnly = FALSE;
	m_SinFreq = 1000;
	m_TimeLimit = 2;
	m_SourceGroup = 4;
	m_OutputGroup = 0;
	m_PSKmode = 0;
	m_SSTVMode = 0;
	m_SNRatio = 10.0;//MAX_SNR;
	m_IMD = -50.0;
	m_FileName = _T("*.wav");
	m_FileSize = _T("");
	m_SoundCardSelect = -1;
	m_IMDGain = .05;
	m_PskSpeedGrp = 0;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_RMSOutput = 0;
}

void CWSCGenDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CWSCGenDlg)
	DDX_Control(pDX, IDC_RATETXT, m_SweepRateCtrl);
	DDX_Control(pDX, IDC_RANGETXT, m_SweepRangeCtrl);
	DDX_Control(pDX, IDC_PSKIDLEONLY, m_IdleOnlyCtrl);
	DDX_Control(pDX, IDC_REVERSE, m_ReverseCtrl);
	DDX_Control(pDX, IDC_RMSVALUE, m_RMSCtrl);
	DDX_Control(pDX, IDC_IMD, m_IMDCtrl);
	DDX_Control(pDX, IDC_FS8000, m_FS8000Cntrl);
	DDX_Control(pDX, IDC_FS9600, m_FS9600Cntrl);
	DDX_Control(pDX, IDC_FS11025, m_FS11025Cntrl);
	DDX_Control(pDX, IDC_FS22050, m_FS22050Cntrl);
	DDX_Control(pDX, IDC_FS44100, m_FS44100Cntrl);
	DDX_Control(pDX, IDC_FS48000, m_FS48000Cntrl);
	DDX_Control(pDX, IDC_BWLIMIT, m_BWLimitCntrl);
	DDX_Control(pDX, IDC_SSTV_S1, m_S1Cntrl);
	DDX_Control(pDX, IDC_SSTV_M1, m_M1Cntrl);
	DDX_Control(pDX, IDC_DISKSIZE, m_DiskSizeCntrl);
	DDX_Control(pDX, IDC_STATUS, m_StatusCntrl);
	DDX_Control(pDX, IDC_USESOUNDCARD, m_UseSoundcardCntrl);
	DDX_Control(pDX, IDC_USEWAVEFILE, m_UseWaveFileCntrl);
	DDX_Control(pDX, IDC_BPSK, m_BPSKCntrl);
	DDX_Control(pDX, IDC_QPSK, m_QPSKCntrl);
	DDX_Control(pDX, IDC_IQPSK, m_IQPSKCntrl);
	DDX_Control(pDX, IDC_CWMODE, m_CWMODECntrl);
	DDX_Control(pDX, IDC_SSTV2, m_SSTV2Source);
	DDX_Control(pDX, IDC_PSK31, m_PSK31Source);
	DDX_Control(pDX, IDC_MSK, m_MSKSource);
	DDX_Control(pDX, IDC_MFSK, m_MFSKSource);
	DDX_Control(pDX, IDC_RTTY, m_RTTYSource);
	DDX_Control(pDX, IDC_PSK400A, m_PSK400aSource);
	DDX_Control(pDX, IDC_PSK400B, m_PSK400bSource);
	DDX_Control(pDX, IDC_SINEWAVE, m_SinSource);
	DDX_Control(pDX, IDC_FILESIZE, m_FileSizeCntrl);
	DDX_Control(pDX, IDC_FILENAME, m_FileNameCntrl);
	DDX_Control(pDX, IDC_WAVESETUP, m_WaveSetupCntrl);
	DDX_Control(pDX, IDC_STOP, m_StopButton);
	DDX_Control(pDX, IDC_RUN, m_RunButton);
	DDX_Control(pDX, IDC_TIMELIMIT, m_TimeLimitCntrl);
	DDX_Control(pDX, IDC_SNR, m_SNRCntrl);
	DDX_Control(pDX, IDC_SINFREQ, m_SinFreqCntrl);
	DDX_Check(pDX, IDC_REVERSE, m_Reverse);
	DDX_Check(pDX, IDC_BWLIMIT, m_BWLimit);
	DDX_Check(pDX, IDC_PSKIDLEONLY, m_PskIdleOnly);
	DDX_Text(pDX, IDC_SINFREQ, m_SinFreq);
	DDX_Text(pDX, IDC_TIMELIMIT, m_TimeLimit);
	DDV_MinMaxUInt(pDX, m_TimeLimit, 1, 5400);
	DDX_Radio(pDX, IDC_SINEWAVE, m_SourceGroup);
	DDX_Radio(pDX, IDC_USESOUNDCARD, m_OutputGroup);
	DDX_Radio(pDX, IDC_BPSK, m_PSKmode);
	DDX_Radio(pDX, IDC_SSTV_S1, m_SSTVMode);
	DDX_Text(pDX, IDC_SNR, m_SNRatio);
	DDX_Text(pDX, IDC_IMD, m_IMD);
	DDV_MinMaxDouble(pDX, m_IMD, -120., 0.);
	DDX_Text(pDX, IDC_FILENAME, m_FileName);
	DDV_MaxChars(pDX, m_FileName, 30);
	DDX_Text(pDX, IDC_FILESIZE, m_FileSize);
	DDV_MaxChars(pDX, m_FileSize, 30);
	DDX_Radio(pDX, IDC_CARDSEL0, m_SoundCardSelect);
	DDX_Radio(pDX, IDC_PSKSPEED1, m_PskSpeedGrp);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CWSCGenDlg, CDialog)
	//{{AFX_MSG_MAP(CWSCGenDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_RUN, OnRun)
	ON_BN_CLICKED(IDC_STOP, OnStop)
	ON_BN_CLICKED(IDM_ABOUTBOX, OnAboutbox)
	ON_EN_KILLFOCUS(IDC_SINFREQ, OnKillfocusSinfreq)
	ON_EN_KILLFOCUS(IDC_SNR, OnKillfocusSnr)
	ON_EN_KILLFOCUS(IDC_TIMELIMIT, OnKillfocusTimelimit)
	ON_BN_CLICKED(IDC_SINEWAVE, OnSinewave)
	ON_BN_CLICKED(IDC_SSTV2, OnSstv)
	ON_BN_CLICKED(IDC_PSK31, OnPsk31)
	ON_BN_CLICKED(IDC_USESOUNDCARD, OnUsesoundcard)
	ON_BN_CLICKED(IDC_USEWAVEFILE, OnUsewavefile)
	ON_BN_CLICKED(IDC_WAVESETUP, OnWavesetup)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SNRSPIN, OnDeltaposSnrspin)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_BWLIMIT, OnBwlimit)
	ON_BN_CLICKED(IDC_FS22050, OnFs22050)
	ON_BN_CLICKED(IDC_FS44100, OnFs44100)
	ON_BN_CLICKED(IDC_FS48000, OnFs48000)
	ON_BN_CLICKED(IDC_FS11025, OnFs11025)
	ON_BN_CLICKED(IDC_FS9600, OnFs9600)
	ON_BN_CLICKED(IDC_FS8000, OnFs8000)
	ON_NOTIFY(UDN_DELTAPOS, IDC_FREQSPIN, OnDeltaposFreqspin)
	ON_EN_KILLFOCUS(IDC_IMD, OnKillfocusImd)
	ON_NOTIFY(UDN_DELTAPOS, IDC_IMDSPIN, OnDeltaposImdspin)
	ON_BN_CLICKED(IDC_PSK400A, OnPsk400a)
	ON_BN_CLICKED(IDC_PSK400B, OnPsk400b)
	ON_BN_CLICKED(IDC_MSK, OnMsk)
	ON_BN_CLICKED(IDC_MFSK, OnMFSK)
	ON_BN_CLICKED(IDC_RTTY, OnRTTY)
	ON_BN_CLICKED(IDC_PSKIDLEONLY, OnPskidleonly)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SWEEPRANGESPIN, OnDeltaposSweeprangespin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SWEEPRATESPIN, OnDeltaposSweepratespin)
	ON_WM_CLOSE()
   ON_BN_CLICKED(IDC_CHECK1, OnReverse)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWSCGenDlg message handlers
//    All the main initialization of user setup parameters is done here
/////////////////////////////////////////////////////////////////////////////
BOOL CWSCGenDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
//	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// Add extra initialization here
	ReadRegistrySettings();	
	UpdateData(FALSE);		// update edit box data
	m_RunButton.SetIcon(AfxGetApp()->LoadIcon(IDI_RUNICON));
	m_StopButton.SetIcon(AfxGetApp()->LoadIcon(IDI_STOPICON));
	CWave Wave;
	if( (m_OutputGroup == 1) && ( m_PathName != _T("") ) )
	{
		OnUsewavefile();
		m_FreeSpace = Wave.GetDriveFreeSpace(&m_PathName);
		m_FileSelected = TRUE;
		m_RunButton.EnableWindow( TRUE );
		UpdateFileStatus();
		CString str;
		str.Format("Total Free %lu KBs", m_FreeSpace/(LONGLONG)1000); //?? %I64 doesn't work
		m_DiskSizeCntrl.EnableWindow( TRUE );
		m_DiskSizeCntrl.SetWindowText(str);
		m_FileSelected = TRUE;
	}
	else
	{
		m_TimeLimitCntrl.EnableWindow( FALSE );
		m_WaveSetupCntrl.EnableWindow( FALSE );
		m_TimeLimitCntrl.EnableWindow( FALSE );
		m_FileNameCntrl.EnableWindow( FALSE );
		m_FileSizeCntrl.EnableWindow( FALSE );
		m_DiskSizeCntrl.EnableWindow( FALSE );
		m_FileSelected = FALSE;
		m_FreeSpace = 1000000;			//assume initially a meg free
		m_DiskSizeCntrl.SetWindowText(_T(""));
	}
	m_SNRCntrl.EnableWindow( TRUE );
	if(m_SourceGroup != 2 )
	{
		m_IMDCtrl.EnableWindow( FALSE );
	}
	if(m_SourceGroup != 2 && m_SourceGroup != 6 )m_IdleOnlyCtrl.EnableWindow( FALSE );

   if(m_SourceGroup != 6 )m_ReverseCtrl.EnableWindow( FALSE );
	if(m_BWLimit)
		m_BWLimitCntrl.SetCheck(1);
	else
		m_BWLimitCntrl.SetCheck(0);
	switch(	m_SampsPerSec )
	{
		case 8000:
			m_FS8000Cntrl.SetCheck(1);
			OnFs8000();
			break;
		case 9600:
			m_FS9600Cntrl.SetCheck(1);
			OnFs9600();
			break;
		case 11025:
			m_FS11025Cntrl.SetCheck(1);
			OnFs11025();
			break;
		case 22050:
			m_FS22050Cntrl.SetCheck(1);
			OnFs22050();
			break;
		case 44100:
			m_FS44100Cntrl.SetCheck(1);
			OnFs44100();
			break;
		case 48000:
			m_FS48000Cntrl.SetCheck(1);
			OnFs48000();
			break;
	}

	m_pProcThread = NULL;
	CalcSNR();
	CalcIMD();
	UpdateSweepValues();
	m_StatusState = STATUS_IDLE;
	m_LastStatusState = -1;

	m_TimerID = SetTimer( ID_TIMER, 1000, NULL);	//start up status timer
	if( m_TimerID == 0)
		AfxMessageBox( _T("Can't Start Status Timer"),MB_OK );
	vpProcThread = NULL;
	vpProcThread = new CProcThread;
	OnStop();
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CWSCGenDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CWSCGenDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CWSCGenDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

// Called when RUN button hit
void CWSCGenDlg::OnRun() 
{
	UpdateData(TRUE);		// update latest values
	m_StopButton.EnableWindow( TRUE );
	m_RunButton.EnableWindow( FALSE );
	m_SSTV2Source.EnableWindow( FALSE );
	m_PSK31Source.EnableWindow( FALSE );
	m_PSK400aSource.EnableWindow(FALSE);
	m_PSK400bSource.EnableWindow(FALSE);
	m_MSKSource.EnableWindow(FALSE);
	m_MFSKSource.EnableWindow(FALSE);
	m_RTTYSource.EnableWindow(FALSE);
	m_SinSource.EnableWindow( FALSE );
	m_BPSKCntrl.EnableWindow( FALSE );
	m_QPSKCntrl.EnableWindow( FALSE );
	m_IQPSKCntrl.EnableWindow( FALSE );
	m_CWMODECntrl.EnableWindow( FALSE );
	m_S1Cntrl.EnableWindow( FALSE );
	m_M1Cntrl.EnableWindow( FALSE );
	m_UseSoundcardCntrl.EnableWindow( FALSE );
	m_UseWaveFileCntrl.EnableWindow( FALSE );
	m_FS8000Cntrl.EnableWindow(FALSE);
	m_FS9600Cntrl.EnableWindow(FALSE);
	m_FS11025Cntrl.EnableWindow(FALSE);
	m_FS22050Cntrl.EnableWindow(FALSE);
	m_FS44100Cntrl.EnableWindow(FALSE);
	m_FS48000Cntrl.EnableWindow(FALSE);
	GetDlgItem( IDC_PSKSPEED1 )->EnableWindow(FALSE);
	GetDlgItem( IDC_PSKSPEED2 )->EnableWindow(FALSE);
	GetDlgItem( IDC_PSKSPEED3 )->EnableWindow(FALSE);

	if(m_SourceGroup == 1)
		m_SinFreqCntrl.EnableWindow( FALSE );
	vpProcThread->StartThread( this );
	if( m_OutputGroup==1 )
		m_StatusState = STATUS_SAVING;
	else
		m_StatusState = STATUS_RUNNING;
	m_RunningTime = 0;
}

// Called when STOP button hit
void CWSCGenDlg::OnStop() 
{
	vpProcThread->StopThread();
	m_StopButton.EnableWindow( FALSE );
	m_RunButton.EnableWindow( TRUE );
	m_SSTV2Source.EnableWindow( TRUE );
	m_PSK31Source.EnableWindow( TRUE );
	m_PSK400aSource.EnableWindow(TRUE);
	m_PSK400bSource.EnableWindow(TRUE);
	m_MSKSource.EnableWindow(TRUE);
	m_MFSKSource.EnableWindow(TRUE);
	m_RTTYSource.EnableWindow(TRUE);
	m_SinSource.EnableWindow( TRUE );
	m_BPSKCntrl.EnableWindow( TRUE );
	m_QPSKCntrl.EnableWindow( TRUE );
	m_IQPSKCntrl.EnableWindow( TRUE );
	m_CWMODECntrl.EnableWindow( TRUE );
	m_S1Cntrl.EnableWindow( TRUE );
	m_M1Cntrl.EnableWindow( TRUE );
	m_UseSoundcardCntrl.EnableWindow( TRUE );
	m_UseWaveFileCntrl.EnableWindow( TRUE );
	m_FS8000Cntrl.EnableWindow(TRUE);
	m_FS9600Cntrl.EnableWindow(TRUE);
	m_FS11025Cntrl.EnableWindow(TRUE);
	m_FS22050Cntrl.EnableWindow(TRUE);
	m_FS44100Cntrl.EnableWindow(TRUE);
	m_FS48000Cntrl.EnableWindow(TRUE);
	GetDlgItem( IDC_PSKSPEED1 )->EnableWindow(TRUE);
	GetDlgItem( IDC_PSKSPEED2 )->EnableWindow(TRUE);
	GetDlgItem( IDC_PSKSPEED3 )->EnableWindow(TRUE);

	if( m_SourceGroup == 1 )
		m_SinFreqCntrl.EnableWindow( FALSE );
	else
		m_SinFreqCntrl.EnableWindow( TRUE );
	m_StatusState = STATUS_IDLE;

}

//Called from process worker thread if it finishes for some reason
void CWSCGenDlg::ProcessAbort()
{
	m_StatusState = STATUS_PROCFINISHED;
}

// Called when About button hit
void CWSCGenDlg::OnAboutbox() 
{
	CAboutDlg dlgAbout;
	dlgAbout.DoModal();
}

// Called when return key pressed
void CWSCGenDlg::OnOK() 
{
	// Perform data range check using the DDX controls
	UpdateData(TRUE);		// update edit boxes with latest values
	CalcSNR();
	CalcIMD();
	UpdateFileStatus();
	ChangeFrequency();
	// Don't let a carriage return cause the program to exit
//	CDialog::OnOK();
}

// Called when sin frequency edit box loses focus
void CWSCGenDlg::OnKillfocusSinfreq() 
{
	UpdateData(TRUE);		// update edit box with latest value
	ChangeFrequency();
}

// Called when SNR edit box loses focus
void CWSCGenDlg::OnKillfocusSnr() 
{
	UpdateData(TRUE);		// update edit box with latest value
	CalcSNR();
}

//Called to enable/disable idle character only pskmode
void CWSCGenDlg::OnPskidleonly() 
{
	UpdateData(TRUE);		// update edit box with latest value
	
}

// Called when timelimit edit box loses focus
void CWSCGenDlg::OnKillfocusTimelimit() 
{
	// Perform data range check using the DDX controls
	UpdateData(TRUE);		// update edit box with latest value
	UpdateFileStatus();
}

// Called when sin wave radio button selected
void CWSCGenDlg::OnSinewave() 
{
	UpdateData(TRUE);		// update edit data with new value
	m_SinFreqCntrl.EnableWindow( TRUE );
	UpdateFileStatus();
	ChangeFrequency();
	m_IMDCtrl.EnableWindow( FALSE );
	m_IdleOnlyCtrl.EnableWindow( FALSE );
   m_ReverseCtrl.EnableWindow( FALSE );
}

// Called when SSTV radio button selected
void CWSCGenDlg::OnSstv() 
{
	UpdateData(TRUE);		// update edit data with new value
	m_SinFreqCntrl.EnableWindow( FALSE );
	UpdateFileStatus();
	m_IMDCtrl.EnableWindow( FALSE );
	m_IdleOnlyCtrl.EnableWindow( FALSE );
   m_ReverseCtrl.EnableWindow( FALSE );
}

// Called when PSK radio button selected
void CWSCGenDlg::OnPsk31() 
{
	UpdateData(TRUE);		// update edit data with new value
	m_SinFreqCntrl.EnableWindow( TRUE );
	UpdateFileStatus();
	m_IMDCtrl.EnableWindow( TRUE );
	m_IdleOnlyCtrl.EnableWindow( TRUE );
   m_ReverseCtrl.EnableWindow( FALSE );

}
// Called when PSK400a radio button selected
void CWSCGenDlg::OnPsk400a() 
{
	UpdateData(TRUE);		// update edit data with new value
	m_SinFreqCntrl.EnableWindow( TRUE );
	UpdateFileStatus();
	m_IMDCtrl.EnableWindow( FALSE );
	m_IdleOnlyCtrl.EnableWindow( FALSE );
   m_ReverseCtrl.EnableWindow( FALSE );
}

// Called when PSK400a radio button selected
void CWSCGenDlg::OnPsk400b() 
{
	UpdateData(TRUE);		// update edit data with new value
	m_SinFreqCntrl.EnableWindow( TRUE );
	UpdateFileStatus();
	m_IMDCtrl.EnableWindow( FALSE );
	m_IdleOnlyCtrl.EnableWindow( FALSE );
   m_ReverseCtrl.EnableWindow( FALSE );

}

// Called when MSK radio button selected
void CWSCGenDlg::OnMsk() 
{
	UpdateData(TRUE);		// update edit data with new value
	m_SinFreqCntrl.EnableWindow( TRUE );
	UpdateFileStatus();
	m_IMDCtrl.EnableWindow( FALSE );
	m_IdleOnlyCtrl.EnableWindow( FALSE );
   m_ReverseCtrl.EnableWindow( FALSE );
}

// Called when MFSK radio button selected
void CWSCGenDlg::OnMFSK() 
{
	UpdateData(TRUE);		// update edit data with new value
	m_SinFreqCntrl.EnableWindow( TRUE );
	UpdateFileStatus();
	m_IMDCtrl.EnableWindow( FALSE );
	m_IdleOnlyCtrl.EnableWindow( TRUE );
   m_ReverseCtrl.EnableWindow( TRUE );
}

// Called when RTTY radio button selected
void CWSCGenDlg::OnRTTY() 
{
	UpdateData(TRUE);		// update edit data with new value
	m_SinFreqCntrl.EnableWindow( TRUE );
	UpdateFileStatus();
	m_IMDCtrl.EnableWindow( FALSE );
	m_IdleOnlyCtrl.EnableWindow( TRUE );
   m_ReverseCtrl.EnableWindow( TRUE );
}

// Called when Soundcard select radio button selected
void CWSCGenDlg::OnUsesoundcard() 
{
	UpdateData(TRUE);		// update with latest value
	m_TimeLimitCntrl.EnableWindow( FALSE );
	m_WaveSetupCntrl.EnableWindow( FALSE );
	m_FileNameCntrl.EnableWindow( FALSE );
	m_FileSizeCntrl.EnableWindow( FALSE );
	m_DiskSizeCntrl.EnableWindow( FALSE );
	m_StopButton.EnableWindow( FALSE );
	m_RunButton.EnableWindow( TRUE );
}

// Called when Wavefile select radio button selected
void CWSCGenDlg::OnUsewavefile() 
{
	UpdateData(TRUE);		// update with latest value
	m_TimeLimitCntrl.EnableWindow( TRUE );
	m_WaveSetupCntrl.EnableWindow( TRUE );
	m_FileNameCntrl.EnableWindow( TRUE );
	m_FileSizeCntrl.EnableWindow( TRUE );
	m_DiskSizeCntrl.EnableWindow( TRUE );
	m_StopButton.EnableWindow( FALSE );
	CWave Wave;
	if(  m_PathName != _T("")  )
	{
		m_FreeSpace = Wave.GetDriveFreeSpace(&m_PathName);
		m_RunButton.EnableWindow( TRUE );
		CString str;
		str.Format("Total Free %lu KBs", m_FreeSpace/(LONGLONG)1000); //?? %I64 doesn't work
		m_DiskSizeCntrl.EnableWindow( TRUE );
		m_DiskSizeCntrl.SetWindowText(str);
		m_FileSelected = TRUE;
	}
	else
	{
		m_FileSelected = FALSE;
		m_RunButton.EnableWindow( FALSE );
	}
	UpdateFileStatus();
}

// Called to open wavefile select dialog
void CWSCGenDlg::OnWavesetup() 
{
//  handler code here for getting Output filename
	m_FileSize = _T("");
	CString sFilter = "Wave Files (*.wav)|*.wav||";
	CFileDialog DlgOutfile( TRUE, "wav", m_FileName, 
			OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT,
			(LPCTSTR)sFilter, this );
	if( DlgOutfile.DoModal() == IDOK)
	{
		m_PathName = DlgOutfile.GetPathName();
		m_FileName = DlgOutfile.GetFileName();
		CWave Wave;
		m_FreeSpace = Wave.GetDriveFreeSpace(&m_PathName);
		m_FileSelected = TRUE;
		m_RunButton.EnableWindow( TRUE );
		UpdateFileStatus();
		CString str;
		str.Format("Total Free %lu KBs", m_FreeSpace/(LONGLONG)1000); //?? %I64 doesn't work
		m_DiskSizeCntrl.SetWindowText(str);
	}else
	{
		if( m_FileSize == _T("") )
		{
			m_FileName = _T("*.wav");
		}
		m_FileSelected = FALSE;
		m_RunButton.EnableWindow( FALSE );
	}
	m_FileNameCntrl.SetWindowText(m_FileName);
	m_FileSizeCntrl.SetWindowText(m_FileSize);
}

//here when cancel button or system exit pressed
void CWSCGenDlg::OnCancel() 
{
	vpProcThread->StopThread( );
	if( m_TimerID != 0 )			// stop timer if running
	{
		KillTimer( m_TimerID );
		m_TimerID = 0;
	}
	if(vpProcThread)
	{
		delete vpProcThread;
		vpProcThread = NULL;
	}
	UpdateData(TRUE);		// update control data
	WriteRegistrySettings();	
	CDialog::OnCancel();
}

// Calculate the signal and noise amplitudes based on SNR in dB
void CWSCGenDlg::CalcSNR()
{
	if( m_SNRatio > MAX_SNR )
		m_SNRatio = MAX_SNR;
	if( m_SNRatio < MIN_SNR )
		m_SNRatio = MIN_SNR;
	UpdateData(FALSE);		// update edit box to new value
	EnterCriticalSection(&vCriticalSection);
	if( m_SNRatio >= 0.0 )
	{
		m_SignalAmplitude = RMS_AMPLITUDE;
		m_NoiseAmplitude = RMS_AMPLITUDE/pow(10.0 , m_SNRatio/20.0);
	}
	else
	{
		m_NoiseAmplitude = RMS_AMPLITUDE;
		m_SignalAmplitude = RMS_AMPLITUDE*pow(10.0 , m_SNRatio/20.0);
	}
	LeaveCriticalSection(&vCriticalSection);
}

// called to recalculate the amplitude value for the IMD test signal
// from the dB value.
void CWSCGenDlg::CalcIMD()
{
	UpdateData(TRUE);		// update edit value from control
	EnterCriticalSection(&vCriticalSection);
	m_IMDGain = BPSK_CONSTANT*pow(10.0 , m_IMD/20.0);;
	LeaveCriticalSection(&vCriticalSection);
}

// called when an inc or dec arrow is pressed on the SNR spin control
void CWSCGenDlg::OnDeltaposSnrspin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
INT temp;
	UpdateData(TRUE);			// get latest SNR value from edit box
	temp = (INT)(m_SNRatio);
	temp = temp - pNMUpDown->iDelta;
	m_SNRatio = (double)temp;
	CalcSNR();
	*pResult = 0;
}

//Called to update the wave file size status if sample size changes
void CWSCGenDlg::UpdateFileStatus()
{
	if( m_OutputGroup==1 )	//if wavefile is output device
	{
		if( (LONGLONG)(m_TimeLimit*m_SampsPerSec*2) >= m_FreeSpace )
		{
			AfxMessageBox(_T("Not Enough Disk Space\nReducing Timelimit"));
			m_TimeLimit = (INT)((m_FreeSpace/(LONGLONG)(m_SampsPerSec*2))-1 );
			if(m_TimeLimit <= 0 )
			{
				m_TimeLimit = 10;
				m_FileName = _T("*.wav");
				m_FileSelected = FALSE;
				m_RunButton.EnableWindow( FALSE );
			}
			UpdateData(FALSE);		// update edit box to new value
		}
		m_FileSize.Format( "%lu KB", (m_TimeLimit*m_SampsPerSec*2)/1000);
		m_FileSizeCntrl.SetWindowText(m_FileSize);
	}
}


//Called when Timer times out to update status
void CWSCGenDlg::OnTimer(UINT nIDEvent) 
{
	if( nIDEvent == m_TimerID )
	{
		UpdateStatus();
	}
	CDialog::OnTimer(nIDEvent);
}

//update current program time or file writing status
void CWSCGenDlg::UpdateStatus()
{
	if( m_StatusState != m_LastStatusState )
	{
		CString Msg;
		CString Msg2;
		switch(	m_StatusState )
		{
			case STATUS_PROCFINISHED:
				m_StopButton.EnableWindow( FALSE );
				m_RunButton.EnableWindow( TRUE );
				m_SSTV2Source.EnableWindow( TRUE );
				m_PSK31Source.EnableWindow( TRUE );
				m_PSK400aSource.EnableWindow(TRUE);
				m_PSK400bSource.EnableWindow(TRUE);
				m_MSKSource.EnableWindow(TRUE);
				m_MFSKSource.EnableWindow(TRUE);
				m_RTTYSource.EnableWindow(TRUE);
				m_SinSource.EnableWindow( TRUE );
				m_BPSKCntrl.EnableWindow( TRUE );
				m_QPSKCntrl.EnableWindow( TRUE );
				m_IQPSKCntrl.EnableWindow( TRUE );
				m_CWMODECntrl.EnableWindow( TRUE );
				m_S1Cntrl.EnableWindow( TRUE );
				m_M1Cntrl.EnableWindow( TRUE );
				m_UseSoundcardCntrl.EnableWindow( TRUE );
				m_UseWaveFileCntrl.EnableWindow( TRUE );
				m_FS8000Cntrl.EnableWindow(TRUE);
				m_FS9600Cntrl.EnableWindow(TRUE);
				m_FS11025Cntrl.EnableWindow(TRUE);
				m_FS22050Cntrl.EnableWindow(TRUE);
				m_FS44100Cntrl.EnableWindow(TRUE);
				m_FS48000Cntrl.EnableWindow(TRUE);
				GetDlgItem( IDC_PSKSPEED1 )->EnableWindow(TRUE);
				GetDlgItem( IDC_PSKSPEED2 )->EnableWindow(TRUE);
				GetDlgItem( IDC_PSKSPEED3 )->EnableWindow(TRUE);

				if( m_SourceGroup == 1 )
					m_SinFreqCntrl.EnableWindow( FALSE );
				else
					m_SinFreqCntrl.EnableWindow( TRUE );
				Msg = _T("Idle");
				m_StatusState = STATUS_IDLE;
				break;
			case STATUS_IDLE:
				Msg = _T("Idle");
				m_LastStatusState = STATUS_IDLE;
				break;
			case STATUS_RUNNING:
				EnterCriticalSection(&vCriticalSection);
				Msg.Format("Elapsed Time = %d Secs", m_RunningTime);
				Msg2.Format("%d", m_RMSOutput);
				m_RunningTime++;
				LeaveCriticalSection(&vCriticalSection);
				m_RMSCtrl.SetWindowText(Msg2);
				m_LastStatusState = -1;
				break;
			case STATUS_SAVING:
				EnterCriticalSection(&vCriticalSection);
				Msg.Format("Writing File, %%%d Done", 100*(m_TimeLimit - m_RunningTime)
													/m_TimeLimit );
				LeaveCriticalSection(&vCriticalSection);
				m_LastStatusState = -1;
				break;
		}
		m_StatusCntrl.SetWindowText(Msg);
	}
}


void CWSCGenDlg::OnBwlimit() 
{
	UpdateData(TRUE);		// update edit box data
}


void CWSCGenDlg::OnFs48000() 
{
	UpdateData(TRUE);		// update edit box data
	m_BWLimit = FALSE;
	m_BWLimitCntrl.EnableWindow(FALSE);
	m_SampsPerSec = 48000;
	UpdateFileStatus();
	ChangeFrequency();
}
void CWSCGenDlg::OnFs44100() 
{
	UpdateData(TRUE);		// update edit box data
	m_BWLimit = FALSE;
	m_BWLimitCntrl.EnableWindow(FALSE);
	m_SampsPerSec = 44100;
	UpdateFileStatus();
	ChangeFrequency();
}
void CWSCGenDlg::OnFs22050() 
{
	UpdateData(TRUE);		// update edit box data
	m_BWLimit = FALSE;
	m_BWLimitCntrl.EnableWindow(FALSE);
	m_SampsPerSec = 22050;
	UpdateFileStatus();
	ChangeFrequency();
}
void CWSCGenDlg::OnFs11025() 
{
	UpdateData(TRUE);		// update edit box data
	m_SampsPerSec = 11025;
	m_BWLimit = FALSE;
	m_BWLimitCntrl.EnableWindow(FALSE);
	UpdateFileStatus();
	ChangeFrequency();
}

void CWSCGenDlg::OnFs9600() 
{
	UpdateData(TRUE);		// update edit box data
	m_BWLimitCntrl.EnableWindow(FALSE);
	m_SampsPerSec = 9600;
	UpdateFileStatus();
	ChangeFrequency();
}

void CWSCGenDlg::OnFs8000() 
{
	UpdateData(TRUE);		// update edit box data
	m_BWLimitCntrl.EnableWindow(TRUE);
	m_SampsPerSec = 8000;
	UpdateFileStatus();
	ChangeFrequency();
}

void CWSCGenDlg::ChangeFrequency()
{
	m_SinFreqLimit = (45*m_SampsPerSec)/100;	//limit sine freq to 90% Fs/2
	if( m_SinFreq > m_SinFreqLimit )
		m_SinFreq = m_SinFreqLimit;
	if( m_SinFreq < MIN_FREQ )
		m_SinFreq = MIN_FREQ;
	UpdateData(FALSE);		// update edit box to new value
}

void CWSCGenDlg::OnDeltaposFreqspin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	UpdateData(TRUE);			// get latest freq value from edit box
	m_SinFreq = m_SinFreq - pNMUpDown->iDelta;
	ChangeFrequency();
	*pResult = 0;
}

void CWSCGenDlg::OnKillfocusImd() 
{
	CalcIMD();	
}

void CWSCGenDlg::OnDeltaposImdspin(NMHDR* pNMHDR, LRESULT* pResult) 
{
INT temp;
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	UpdateData(TRUE);			// get latest IMD value from edit box
	if(m_SourceGroup == 2)
	{
		temp = (INT)(m_IMD);
		temp = temp - pNMUpDown->iDelta;
		m_IMD = 1.0 * (double)temp;
		if(m_IMD>0.0)
			m_IMD = 0.0;
		if(m_IMD < -120.0 )
			m_IMD = -120.0;
		UpdateData(FALSE);			// put latest IMD value in edit box
		CalcIMD();
	}
	
	*pResult = 0;
}


BOOL CWSCGenDlg::WriteRegistrySettings()
{
CWinApp* pApp = AfxGetApp();
CString str;
//save BOOL's
	str.Format("%i", m_BWLimit);
	if(!pApp->WriteProfileString("Settings\\m_BWLimit", "m_BWLimit", str))
		return FALSE;
	str.Format("%i", m_PskIdleOnly);
	if(!pApp->WriteProfileString("Settings\\m_PskIdleOnly", "m_PskIdleOnly", str))
		return FALSE;
	

//save INT's
	str.Format("%i", m_SourceGroup );
	if(!pApp->WriteProfileString("Settings\\m_SourceGroup", "m_SourceGroup", str) )
		return FALSE;
	str.Format("%i", m_OutputGroup );
	if(!pApp->WriteProfileString("Settings\\m_OutputGroup", "m_OutputGroup", str) )
		return FALSE;
	str.Format("%i", m_SinFreq );
	if(!pApp->WriteProfileString("Settings\\m_SinFreq", "m_SinFreq", str) )
		return FALSE;
	str.Format("%i", m_TimeLimit );
	if(!pApp->WriteProfileString("Settings\\m_TimeLimit", "m_TimeLimit", str) )
		return FALSE;
	str.Format("%i", m_PSKmode );
	if(!pApp->WriteProfileString("Settings\\m_PSKmode", "m_PSKmode", str) )
		return FALSE;
	str.Format("%i", m_SSTVMode );
	if(!pApp->WriteProfileString("Settings\\m_SSTVMode", "m_SSTVMode", str) )
		return FALSE;
	str.Format("%i", m_SampsPerSec );
	if(!pApp->WriteProfileString("Settings\\m_SampsPerSec", "m_SampsPerSec", str) )
		return FALSE;
	str.Format("%i", m_SweepRange );
	if(!pApp->WriteProfileString("Settings\\m_SweepRange", "m_SweepRange", str) )
		return FALSE;
	str.Format("%i", m_SweepRate );
	if(!pApp->WriteProfileString("Settings\\m_SweepRate", "m_SweepRate", str) )
		return FALSE;
	str.Format("%i", m_SoundCardSelect );
	if(!pApp->WriteProfileString("Settings\\m_SoundCardSelect", "m_SoundCardSelect", str) )
		return FALSE;
	str.Format("%i", m_PskSpeedGrp);
	if(!pApp->WriteProfileString("Settings\\m_PskSpeedGrp", "m_PskSpeedGrp", str))
		return FALSE;
	
//save doubles's
	str.Format("%lg",m_SNRatio);
	if(!pApp->WriteProfileString("Settings\\m_SNRatio", "m_SNRatio", str) )
		return FALSE;
	str.Format("%lg",m_IMD);
	if(!pApp->WriteProfileString("Settings\\m_IMD", "m_IMD", str) )
		return FALSE;

//save strings
	if(!pApp->WriteProfileString("Settings\\Strings", "m_PathName", m_PathName))
		return FALSE;
	if(!pApp->WriteProfileString("Settings\\Strings", "m_FileName", m_FileName))
		return FALSE;
	if(!pApp->WriteProfileString("Settings\\Strings", "m_FileSize", m_FileSize))
		return FALSE;
	return TRUE;
}

BOOL CWSCGenDlg::ReadRegistrySettings()
{
CWinApp* pApp = AfxGetApp();
CString str;
INT cRead;
// Get BOOLs
	str = pApp->GetProfileString ("Settings\\m_BWLimit", "m_BWLimit");
	cRead = _stscanf (str, "%i", &m_BWLimit );
	if(cRead != 1)
		m_BWLimit = TRUE;
	str = pApp->GetProfileString ("Settings\\m_PskIdleOnly", "m_PskIdleOnly");
	cRead = _stscanf (str, "%i", &m_PskIdleOnly );
	if(cRead != 1)
		m_PskIdleOnly = FALSE;


//Get INT's
	str = pApp->GetProfileString ("Settings\\m_SourceGroup", "m_SourceGroup");
	cRead = _stscanf (str, "%i", &m_SourceGroup );
	if(cRead != 1)
		m_SourceGroup = 0;
	str = pApp->GetProfileString ("Settings\\m_OutputGroup", "m_OutputGroup");
	cRead = _stscanf (str, "%i", &m_OutputGroup );
	if(cRead != 1)
		m_OutputGroup = 0;
	str = pApp->GetProfileString ("Settings\\m_SinFreq", "m_SinFreq");
	cRead = _stscanf (str, "%i", &m_SinFreq );
	if(cRead != 1)
		m_SinFreq = 1000;
	str = pApp->GetProfileString ("Settings\\m_TimeLimit", "m_TimeLimit");
	cRead = _stscanf (str, "%i", &m_TimeLimit );
	if(cRead != 1)
		m_TimeLimit = 2;
	str = pApp->GetProfileString ("Settings\\m_PSKmode", "m_PSKmode");
	cRead = _stscanf (str, "%i", &m_PSKmode );
	if(cRead != 1)
		m_PSKmode = 0;
	str = pApp->GetProfileString ("Settings\\m_SSTVMode", "m_SSTVMode");
	cRead = _stscanf (str, "%i", &m_SSTVMode );
	if(cRead != 1)
		m_SSTVMode = 0;
	str = pApp->GetProfileString ("Settings\\m_SampsPerSec", "m_SampsPerSec");
	cRead = _stscanf (str, "%i", &m_SampsPerSec );
	if(cRead != 1)
		m_SampsPerSec = 8000;
	str = pApp->GetProfileString ("Settings\\m_SweepRange", "m_SweepRange");
	cRead = _stscanf (str, "%i", &m_SweepRange );
	if(cRead != 1)
		m_SweepRange = 0;
	str = pApp->GetProfileString ("Settings\\m_SweepRate", "m_SweepRate");
	cRead = _stscanf (str, "%i", &m_SweepRate );
	if(cRead != 1)
		m_SweepRate = 0;
	str = pApp->GetProfileString ("Settings\\m_SoundCardSelect", "m_SoundCardSelect");
	cRead = _stscanf (str, "%i", &m_SoundCardSelect );
	if(cRead != 1)
		m_SoundCardSelect = 0;
	str = pApp->GetProfileString ("Settings\\m_PskSpeedGrp", "m_PskSpeedGrp");
	cRead = _stscanf (str, "%i", &m_PskSpeedGrp );
	if(cRead != 1)
		m_PskSpeedGrp = 0;

//get doubles's
	str = pApp->GetProfileString("Settings\\m_SNRatio", "m_SNRatio");
	cRead = _stscanf(str, "%lg",&m_SNRatio );
	if(cRead != 1)
		m_SNRatio = 20.0;
	str = pApp->GetProfileString("Settings\\m_IMD", "m_IMD");
	cRead = _stscanf(str, "%lg",&m_IMD );
	if(cRead != 1)
		m_IMD = -60.0;
//get strings
	m_PathName = pApp->GetProfileString("Settings\\Strings", "m_PathName",_T(""));
	m_FileName = pApp->GetProfileString("Settings\\Strings", "m_FileName",_T(""));
	m_FileSize = pApp->GetProfileString("Settings\\Strings", "m_FileSize",_T(""));
	return TRUE;
}



void CWSCGenDlg::OnDeltaposSweeprangespin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	UpdateData(TRUE);			// get latest freq value from edit box
	m_SweepRange = m_SweepRange - pNMUpDown->iDelta;
	UpdateSweepValues();	
	*pResult = 0;
}

void CWSCGenDlg::OnDeltaposSweepratespin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	UpdateData(TRUE);			// get latest freq value from edit box
	m_SweepRate = m_SweepRate - pNMUpDown->iDelta;
	UpdateSweepValues();	
	*pResult = 0;
}

void CWSCGenDlg::UpdateSweepValues()
{
CString str;
	if( m_SweepRate > 1000 )
		m_SweepRate = 1000;
	if( m_SweepRate < 0 )
		m_SweepRate = 0;
	if( m_SweepRange > (INT)m_SinFreqLimit-(INT)m_SinFreq )
		m_SweepRange = (INT)m_SinFreqLimit-(INT)m_SinFreq;
	if( m_SweepRange > (INT)m_SinFreq-MIN_FREQ )
		m_SweepRange = (INT)m_SinFreq-MIN_FREQ;
	if( m_SweepRange < 0 )
		m_SweepRange = 0;
	str.Format("%d",m_SweepRate);
	m_SweepRateCtrl.SetWindowText(str);
	str.Format("%d",m_SweepRange);
	m_SweepRangeCtrl.SetWindowText(str);

}



void CWSCGenDlg::OnReverse()
{
   UpdateData(TRUE);
}
